@@ -3,3 +3,5 @@ from .bruteForce import BruteForce
 from .gula import GULA
 from .pride import PRIDE
 from .synchronizer import Synchronizer
+from .probalizer import Probalizer
+from .acedia import ACEDIA
