@@ -2,3 +2,5 @@ from .model import Model
 from .dmvlp import DMVLP
 from .cdmvlp import CDMVLP
 from .wdmvlp import WDMVLP
+from .pdmvlp import PDMVLP
+from .clp import CLP
